<?php

class AutoBlocking {

	/**
	 * Process the given code.
	 *
	 * @param   string  $code  The code to process.
	 *
	 * @return string The processed code.
	 */
	public function processCode( $code ) {
		/**
		 * Attempt to retrieve the site ID from the provided code.
		 * If unsuccessful, do nothing and continue to the next iteration.
		 */
		$siteId = $this->getSiteIdFromCsCode( $code );
		if ( empty( $siteId ) ) {
			return $code;
		}

		/**
		 * Check if the autoblocking feature is available for the site.
		 * Remove existing autoblocking and sync scripts if present.
		 */
		$autoblockingStatus = $this->isAutoblockingFeatureAvailable( $siteId );
		$code               = $this->removeExistingScripts( $code );

		/**
		 * If the autoblocking feature is available and the code doesn't contain an autoblocking script,
		 * append the autoblocking script. Otherwise, if the code doesn't contain a sync script, append the sync script.
		 */
		if ( $autoblockingStatus && ! $this->isCodeContainsAutoblockingScript( $code ) ) {
			$code = $this->preAppendAutoblockingScript( $code, $siteId );
		} elseif ( ! $this->isCodeContainsSyncScript( $code ) ) {
			$code = $this->preAppendSyncScript( $code, $siteId );
		}

		return $code;
	}

	/**
	 * Parses the provided script and extracts the site ID.
	 *
	 * @param   string  $code  The script to parse.
	 *
	 * @return  string  The extracted site ID.
	 */
	public function getSiteIdFromCsCode( $code ) {
		$code                = stripslashes( $code );
		$parsedConfiguration = $this->parseConfigurationByRegex( $code );

		if ( isset( $parsedConfiguration['siteId'] ) ) {
			return $parsedConfiguration['siteId'];
		}

		return '';
	}

	/**
	 * Checks whether the autoblocking feature is available for the given site ID.
	 *
	 * @param   string  $siteId  The site ID to check.
	 *
	 * @return  bool    True if the autoblocking feature is available; otherwise, false.
	 */
	public function isAutoblockingFeatureAvailable( $siteId ) {
		$url = 'https://cs.iubenda.com/autoblocking/' . $siteId . '.js';

		try {
			$response = \Tools::file_get_contents( $url );
		} catch ( \Exception $e ) {
			return false;
		}

		$content = $response;

		return strlen( $content ) > 150 && strpos( $content, "Autoblocking not enabled" ) === false;
	}

	/**
	 * Extracts configuration data from an iubenda code using regex.
	 *
	 * @param   string  $code  The iubenda code.
	 *
	 * @return  array   Extracted configuration data.
	 */
	public function parseConfigurationByRegex( $code ) {
		$result = [];
		$code   = stripslashes( $code );

		// Extract 'siteId'.
		if ( preg_match( '/siteId([\s\S]*?)(?:,|})/', $code, $matches ) ) {
			$result['siteId'] = trim( preg_replace( "/(?:'|\"|}|:)/", ' ', $matches[1] ) );
		}

		// Extract 'cookiePolicyId'.
		if ( preg_match( '/cookiePolicyId([\s\S]*?)(?:,|})/', $code, $matches ) ) {
			$result['cookiePolicyId'] = trim( preg_replace( "/(?:'|\"|}|:)/", ' ', $matches[1] ) );
		}

		return $result;
	}

	/**
	 * Removes existing autoblocking and sync scripts from the given code.
	 *
	 * @param   string  $code  The code to clean.
	 *
	 * @return  string  The code without autoblocking and sync scripts.
	 */
	public function removeExistingScripts( $code ) {
		// Patterns to identify autoblocking and sync scripts.
		$autoblockingPattern = "/<script[^>]*src=['\"].*iubenda\.com\/autoblocking\/[^>]+><\/script>/";
		$syncPattern         = "/<script[^>]*src=['\"].*iubenda\.com\/sync\/[^>]+><\/script>/";

		// Remove the scripts if found.
		$code = preg_replace( $autoblockingPattern, '', $code );
		$code = preg_replace( $syncPattern, '', $code );

		return ltrim( $code );
	}

	/**
	 * Checks if the given code contains the autoblocking script.
	 *
	 * @param   string  $code  The code to check.
	 *
	 * @return  bool    True if the code contains the autoblocking script; otherwise, false.
	 */
	public function isCodeContainsAutoblockingScript( $code ) {
		$code = stripslashes( $code );

		return strpos( $code, "iubenda.com/autoblocking/" ) !== false;
	}

	/**
	 * Prepends the autoblocking script to the given code.
	 *
	 * @param   string  $code     The code to prepend the script to.
	 * @param   string  $site_id  The site ID for the autoblocking script.
	 *
	 * @return  string  The code with the autoblocking script prepended.
	 */
	public function preAppendAutoblockingScript( $code, $site_id ) {
		$autoblockingScriptUrl = "<script type='text/javascript' src='https://cs.iubenda.com/autoblocking/{$site_id}.js' charset='UTF-8' async></script>";

		return $autoblockingScriptUrl . PHP_EOL . $code;
	}

	/**
	 * Checks if the given code contains the sync script.
	 *
	 * @param   string  $code  The code to check.
	 *
	 * @return  bool    True if the code contains the sync script; otherwise, false.
	 */
	public function isCodeContainsSyncScript( $code ) {
		$code = stripslashes( $code );

		return strpos( $code, "iubenda.com/sync/" ) !== false;
	}

	/**
	 * Prepends the sync script to the given code.
	 *
	 * @param   string  $code     The code to prepend the script to.
	 * @param   string  $site_id  The site ID for the sync script.
	 *
	 * @return  string  The code with the sync script prepended.
	 */
	public function preAppendSyncScript( $code, $site_id ) {
		$syncScriptUrl = "<script type='text/javascript' src='https://cs.iubenda.com/sync/{$site_id}.js' charset='UTF-8' async></script>";

		return $syncScriptUrl . PHP_EOL . $code;
	}

}
